/* Functions: barscalefile() and do_barscale()
**
** Author: Horst Raap	April 2000
*/

#include "ps_info.h"
#include "barscale.h"


#define KEY(x) (strcmp(key,x)==0)

static char *help[] =
{
    "where      x y",
    "unit       feet/km/miles/meter",
    "length     x",
    "width      x",
    "interval   x",
    "style      tick/slash",
    "font       fontname",
    "fontsize   fontsize",
    "textcolor  c",
    "color      c",
    "background c",
    "border     c",
    ""
};

barscalefile()
{	
    char buf[1024];
    char *key, *data;

    /* The defaults */
    barscale.x = -1.0;
    barscale.y = -1.0;
    barscale.unit = 1.0;
    barscale.unitname = "m";
    barscale.length = 100;
    barscale.interval = 25;
    barscale.style = 0;   /* means dash */
    barscale.width = 8;
    barscale.color = BLACK;
    barscale.textcolor = BLACK;
    barscale.fontsize = 10;
    barscale.font = "Helvetica";  
    barscale.background = -1;
    barscale.border = -1;
 
    while (input(2, buf, help))
    {
	if (!key_data(buf, &key, &data)) continue;

        if (KEY("where"))
 	{
	    scan_where(data, &barscale.x, &barscale.y);
	    continue;
	}

        if (KEY("unit"))
          {
            if (*data == 'f') 
              { 
              barscale.unitname = "ft";
              barscale.unit = 0.3048;  
              }
            if (*data == 'k')
              {
               barscale.unitname = "km";
               barscale.unit = 1000.0;
              };
               
            if (*(data+1) == 'i') 
              {
               barscale.unitname="mi";
               barscale.unit = 1609.3;
              };            
            continue;
          };
 
        if (KEY("length"))
          {
           barscale.length=atoi(data); 
           continue;
          }; 

        if (KEY("interval"))
           {
            barscale.interval=atoi(data);
            continue;
            };

        if (KEY("style"))
         {
          if (*data == 't') barscale.style=1; 
          continue; 
         };
 
	if (KEY("fontsize"))
	{
	    barscale.fontsize = atoi(data);
	    if (barscale.fontsize < 4 || barscale.fontsize > 50) 
               barscale.fontsize = 0;
	    continue;
	}

        if (KEY("width"))
          {
           sscanf(data, "%lf", &barscale.width);
           continue;
          };
  

	if (KEY("font"))
	{
	    get_font(data);
	    barscale.font = G_store(data);
	    continue;
	}

        if (KEY("color"))
          {
            barscale.color = get_color_number(data);
            continue;
          };

        if (KEY("textcolor"))
          {
            barscale.textcolor = get_color_number(data);
            continue;
          };

        if (KEY("background"))
          {
            barscale.background = get_color_number(data);
            continue;
          };

        if (KEY("color"))
          {
            barscale.border = get_color_number(data);
            continue;
          };

        if (KEY("border"))
          {
            barscale.border = get_color_number(data);
            continue;
          };

	error(key, data, "illegal barscale sub-request");
    }

}


void
do_barscale()
{
    double fontsize=barscale.fontsize;
    double dy, t, l;
    double east, north;
    
    double x1, y1, x, y;
    double height;
    int i, ninterval, color;
    double length, inter, pslength; 
    char *font="Helvetica";

    fprintf(PS.fp, "gsave\n"); 
 
    /* set barscale location */
    if (barscale.y <= 0.0) t = PS.min_y;
    else t = 72.0 * ( PS.page_height - barscale.y);
    if (barscale.x <= 0.0) barscale.x = PS.left_marg;
    l = 72.0 * barscale.x + 0.5;

    dy = 2.0 * fontsize;

    /* Make sure interval divides length evenly ! */
    if (barscale.length % barscale.interval != 0)  
      return;
    /* The length variables in meters */
    length = barscale.length * barscale.unit;
    inter=barscale.interval * barscale.unit; 
    ninterval = barscale.length / barscale.interval;

    /* Length in postscript units */
    pslength = length * PS.ew_to_x;

    /* The line width */ 
    fprintf(PS.fp,"0.5 W\n");

    /* Starting point in xy -coordinates of border box, upper left point*/  
    x=l;
    y=t;
    height= 0.75 * fontsize;

   /* Lower point of graphics */
   y1 = y -fontsize;

    /* Let Postscript compute four values:
       le = length of " 0" left of barscale (left extension) 
       re = length of length string right of barscale (right extension)
       ue = length of unit string (incl. spaces) (unit extension) 
       xs = starting x-coordinate of ticks or dashes
       Numbers are in smaller font 
    */
    fprintf(PS.fp,"gsave\n"); 
    fprintf(PS.fp," (%s) FN %.2lf 2.0 div  SF \n", font, fontsize);
    fprintf(PS.fp,"/le ( 0) stringwidth pop 2.0 div def\n");  
    fprintf(PS.fp,"/re (%ld) stringwidth pop 2.0 div def\n", (int)(barscale.length+0.5));  
    fprintf(PS.fp," (%s) FN %.2lf SF \n", font, fontsize);
    fprintf(PS.fp,"/ue ( %s ) stringwidth pop def\n", barscale.unitname);
    fprintf(PS.fp,"/xs %.2lf le add le add def\n", x);
    fprintf(PS.fp,"grestore\n");

    /* Draw the colored background box, if any */ 

    fprintf(PS.fp, "%.2lf %.2lf ",x,y, fontsize);
    fprintf(PS.fp, "%.2lf le add ue re max add %.2lf B ",x+length*PS.ew_to_x, y-2*fontsize); 
    if (barscale.background >= 0)
      {
       set_rgb_color(barscale.background);
       fprintf(PS.fp, "F ");
      };
    /* and maybe outline the box */  
    if (barscale.border >= 0)
    {
      set_rgb_color(barscale.border);
      fprintf(PS.fp, "stroke\n");
    };

    /* Define a postscript procedure to center a text at x coordinate */
    /* calling  text x  y --       shows text center at x */
    fprintf(PS.fp,"/CENTER { 3 -1 roll dup stringwidth pop 2.0 div 
                    4 -1 roll exch sub 3 -1 roll MS } def\n");

    /* Let postscript compute half length of right text = lrt */
    fprintf(PS.fp,"/lrt (%ld) stringwidth pop 2.0 div def\n",(int)(barscale.length+0.5));

    set_rgb_color(barscale.color);  

    if (barscale.style != 0)
      {
      /* Print vertical starting tick */
      fprintf(PS.fp," xs %.2lf xs %.2lf L\n", y1, y1+height);  

      /* Print line length*unit meters long */
      fprintf(PS.fp," xs %.2lf ", y1+height/2.0);
      fprintf(PS.fp," xs %.2lf add %.2lf  L\n", pslength, y1+height/2.0);

      /* Print interval ticks */
      for (i=1; i<ninterval; i++)
        fprintf(PS.fp," xs %.2lf add %.2lf xs %.2lf add  %.2lf L\n", 
                 i*inter*PS.ew_to_x, y1+height/6.0, i*inter*PS.ew_to_x, y1+height*5.0/6.0);  
      
      /* Print vertical stop tick */
      fprintf(PS.fp," xs %.2lf add %.2lf xs %.2lf add %.2lf L\n", 
                 pslength, y1, pslength, y1+height);      
      }
    else
      {
      color = barscale.color;
      for (i=0; i<ninterval; i++)
        {
         /* Draw alternating black and white boxes */
        set_rgb_color(color); 

        fprintf(PS.fp," xs %.2lf add %.2lf ", i*inter*PS.ew_to_x, y1);  
        fprintf(PS.fp," xs %.2lf add %.2lf B fill\n", (i+1)*inter*PS.ew_to_x, y1+height);  
        
        if (color != WHITE) color = WHITE;
        else color = barscale.color;
        };

      /* Draw a thin border */ 
      set_rgb_color(barscale.color); 
      fprintf(PS.fp,"0.3 W "); 
      fprintf(PS.fp," xs %.2lf ", y-fontsize);  
      fprintf(PS.fp," xs %.2lf add %.2lf B stroke\n", length*PS.ew_to_x, y1+height);  
      };

    /* The texts */

    set_rgb_color(barscale.textcolor);

    /* Now for the text in smaller font */
    fprintf(PS.fp," (%s) FN %.2lf 2.0 div  SF \n", font, fontsize);
    fprintf(PS.fp," (%ld) xs %.2lf CENTER \n", 0,y1-fontsize*0.5);
    fprintf(PS.fp," (%ld) xs %.2lf add %.2lf CENTER \n", (int)(barscale.length+0.5),pslength, y1-fontsize*0.5);

    /* The units text */
    fprintf(PS.fp," (%s) FN %.2lf SF \n", font, fontsize);
    fprintf(PS.fp, " ( %s) xs %.2lf add %.2lf MS \n", barscale.unitname, pslength, y1);

    fprintf(PS.fp, "grestore\n");

    /* Remember lowest used y-coordinate */
    y -= dy;
    if (PS.min_y > y) PS.min_y = y;
};

