/* Function: map_setup
**
** Author: Paul W. Carlson	3/92
*
* We know the scale and/or the raster map to be printed
*
* Computes the following values:
*
* --> PS.map_width, PS.map_height
* --> PS.map_pix_wide, PS.map_pix_high
* --> PS.ew_to_x, PS.ns_to_y 
*
* initializes distance calculations and plotting routines
*/

#include "ps_info.h"
#include "group.h"

extern int verbose;
extern int move_local(), cont_local();

map_setup()
{
    char buf[128];
    int row, col, cells_per_inch;
    double w, h, max, aspect;
    long num_cells, limit;

    /* No raster and no group ... */
    if (!PS.do_raster && !grp.do_group)
    {	
	/* if scale has been set... */
    	if (PS.scaletext[0]) 
    	{
	    /* if scaled map will fit in map limits... */
	    w = scale(PS.scaletext);
            /* CHECK: Can you simply subtract these values ?*/ 
	    h = w * (PS.w.north - PS.w.south) / (PS.w.east  - PS.w.west);
	    if (w <= PS.map_width && h <= PS.map_height)
	    {
	    	PS.map_width  = w;
	    	PS.map_height = h;
	    }

	    /* else, kill the scale */
	    else PS.scaletext[0] = 0;
	}
	
	/* fit map to bounding box */
	fit_map_to_box();
    }

    else
    /* raster or group selected */
    {

        /* If user has selected a scale, use it if possible */ 
    	if (PS.scaletext[0]) 
    	{
	    /* if scaled map will fit in map limits... */
	    w = scale(PS.scaletext);

           /* Set height corresponding to width, keep aspect ratio ! */
	    h = w * PS.w.ns_res * (double)PS.w.rows /
    		   (PS.w.ew_res * (double)PS.w.cols);
	    if (w <= PS.map_width && h <= PS.map_height)
	    {
	    	PS.map_width  = w;
	    	PS.map_height = h;
	    }

	    /* else, kill the scale */
	    else PS.scaletext[0] = 0;
	}

	/* fit map to bounding box */
	fit_map_to_box();


        if (verbose > 2)
          {
          printf("%d dots high, %d dots wide\n",PS.dots_high, PS.dots_wide);
          printf("%lf map pix high, %lf map pixels wide\n",PS.map_pix_high, PS.map_pix_wide);
          }

        /* Avoid to create more dots than the limit (file would be too large) */
        /* Allow 300 (100) million b/w (color) postscript pixels */
	if (PS.grey || PS.level == 1) limit = 300000000L;
	else limit = 100000000L;

        /* This is maximum area for map => map_height * map_width: */
	max = (double) limit / (double) (PS.res * PS.res);

	/* Aspect ratio: height / width */
        aspect = PS.map_height / PS.map_width;

        /* So, area = map_width * aspect * map_width */

        /* Now compute maximum width */
        w = sqrt (max / aspect);

        /* adjust if limit exceeded */
        if (PS.map_width > w)
          {
          PS.map_width = w;
          PS.map_height = w * aspect;

          if (verbose > 0)
            printf("PS-MAP: File would be too large. Resolution reduced!\n");
          }; 

        /* Recompute necessary values */
        fit_map_to_box();

        /* CHECK: rounding */
        /* Number of postscript dots */ 
        PS.dots_high = (int) 0.5 + PS.map_height * (double) PS.res;
        PS.dots_wide = (int) 0.5 + PS.map_width * (double) PS.res;

        if (verbose > 2)
          {
          printf("%d pixels high, %d pixels wide\n",PS.dots_high, PS.dots_wide);
          printf("%lf map pix high, %lf map pixels wide\n",PS.map_pix_high, PS.map_pix_wide);
          }
    };

    /* set the scale */
    if (!PS.scaletext[0]) sprintf(PS.scaletext, "1 : %.0lf",
	39.37 * (PS.w.east - PS.w.west) / PS.map_width);
    if (verbose > 1)
    {
        printf("PS-MAP: scale set to %s.\n", PS.scaletext);
        fflush(stdout);
    }

    /* compute map edges */
    PS.map_left  = 72.0 * PS.map_x_orig;
    PS.map_top   = 72.0 * PS.map_y_orig;
    PS.map_bot   = PS.map_top  - PS.map_pix_high;
    PS.map_right = PS.map_left + PS.map_pix_wide;
    PS.min_y = PS.map_bot;

    /* we want the size to be 10 times biger, because G_plot_where_xy()
       returns integer values (pixels) for x and y, and we want doubles
       until the first decimal point. Then in move() and cont() we will
       divide x and y by 10. to get double coordinates */
    G_setup_plot(PS.map_top * 10., PS.map_bot * 10., PS.map_left * 10., PS.map_right * 10., move_local, cont_local);


    /* save original graphics state */
    fprintf(PS.fp, "gsave ");

    
    /* compute conversion factor from meters to PostScript window coordinates */
    /* COMMENTED OUT
    G_begin_distance_calculations();
    meters_to_PS = (PS.map_top - PS.map_bot) / G_distance(0., PS.w.south, 0., PS.w.north);
    */

    /* clip to edge of border */
    box_clip(PS.map_top  - 1.0, PS.map_bot   + 1.0, 
	     PS.map_left + 1.0, PS.map_right - 1.0);
}

