#include "ps_info.h"
#include "pattern.h"


#define KEY(x) (strcmp(key,x)==0)

static char *help[] =
{
    "pattern-line",
    "color      color",
    ""
};



typedef
struct plist 
{
    char *name;
    PATTERN p;
    struct plist *next;
} PLIST;


/* A list of all the patterns defined at the moment */
static PLIST *plist = NULL;


#if 0

static FILE *patternfd = NULL;
static char *patternfile = NULL;
static long cur_offset = 0;
static char *cur_name;
static int nl;
static int nr;
static int nc;
static int cur_file = -1;
static int cur_mode = -1;

#endif

/* Make this a MACRO , speed */
static int
Cat2Patval(PATTERN *ppat, CELL cat, int row, int col)
{
  return (ppat->pat[row % ppat->nrows][col % ppat->ncols]);
};


/* Sets RGB values from pattern-value */
static void
Patval2RGB(PATTERN *ppat, int patval, int *r, int *g, int *b) 
{
  *r = ppat->colors[patval][0];
  *g = ppat->colors[patval][1];
  *b = ppat->colors[patval][2];
};


/*
 * Set the RGB-values for a specific category 
 * Uses pattern if set 
 * The row and column are given in Output-Space
*/ 
void 
get_color(CELL cat, int *red, int *green, int *blue, struct Colors *colors, int row, int col)
{ 
   int patval;
   PATTERN *ppat;

    if (NULL != (ppat=HasPattern(cat)) )
      {  

        /* Get the pattern for this category */
        patval=Cat2Patval(ppat, cat, row, col);
        if (patval!=0) 
        {  
          Patval2RGB(ppat, patval, red, green, blue);         
          return;
        } 

      };

    G_get_color(cat, red, green, blue, colors);
};

/*
  Returns pointer to pattern if the given category has one,
  else returns NULL
*/

PATTERN *
HasPattern(CELL cat)
{
  if (cat > 255) return NULL;
  return(PS.pattable[cat]);
};


/* 
  Add copy of named pattern to plist 
*/
static
add_pattern_to_list (char *name, PATTERN *p)
{
    PLIST *pl;

    /* make copy of name */
    if (name == NULL)
	name = "?????";
    else
	name = G_store (name);


    /* New list element */
    pl = (PLIST *) G_malloc (sizeof(PLIST));

    pl->name   = name;
    pl->p = *p;

    pl->next = plist;
    plist = pl;
}



/*
  find a named pattern in plist 
  returns pointer to pattern if found,
  else returns NULL
*/

static PATTERN *
find_pattern (char *name)
{
    PLIST *pl;

/*
    read_predefined_patterns();
*/


    for (pl = plist; pl; pl = pl->next)
	if (strcmp (name, pl->name) == 0)
	  return &(pl->p);

    return NULL;
}

/* Set a specific named pattern for category cat */
int
set_pattern(CELL cat, char *name)
{
   PATTERN *p;

   p = find_pattern(name);
   
   PS.pattable[cat] = p;
   return 1;
};


/*
  Handle pattern lines with keywords "color"  
  Returns 1 if this was a correctly formed color line
  and sets color structure in pat
  else returns 0
*/
static int 
scan_patcolor(PATTERN *pat, char *buf)
{
  char temp[256], temp2[256] ;
  int n, cn;
  float r, g, b;

   /* Check for keyword "color" */
   sscanf (buf, "%s", temp);
   if (strcmp(temp, "color") != 0) 
      return 0;

   if (sscanf(buf, "%s %d %[^\n]", temp, &n, temp2) != 3)
      return 0;

   if ( (n<0) || (n > 9) )
      return 0; 

   if ((cn=get_color_number (temp2)) < 0) 
     return 0; 

   get_color_rgb(cn, &r, &g, &b);

   pat->colors[n][0]=(int) (r*255.0);
   pat->colors[n][1]=(int) (g*255.0);
   pat->colors[n][2]=(int) (b*255.0);
 
   return 1;
};


/*
 Read pattern from file 
 File is open and positioned at next line after "defpat"
 The resulting pattern is stored under its name in plist
*/

defpatfile(char *name)
{
    PATTERN pat1;
    PATTERN *p = &pat1;

    char buf[1024];
    char temp[256];

    /* Temporarily holds the lines */
    char *lines[256];
 
    char *pc;
    int line, row, col;
    int nrows, ncols;
    int count;
    int i, color;

    /* default color is black */
    for (i=0; i<10; i++)
      {
       p->colors[i][0] = 0;
       p->colors[i][1] = 0;
       p->colors[i][2] = 0;
      };

    /* First compute number of rows and columns */
    ncols=0;
    nrows = 0;

    while (input(2, buf, help))
      {	    
         /* Check for keyword "end" */
         sscanf (buf, "%s", temp);
         if (strcmp(temp, "end") == 0) 
           break;

	/* Handle keyword "color" */
	if (scan_patcolor (p, buf) != 0)
	  continue;

        /* translate characters to 0..9 */
        for (pc=buf; *pc!= '\0'; *pc++)
          { 
           if (*pc == ' ') *pc='0';
          };

        lines[nrows++]=G_store(buf);
        col = strlen(buf);
        if (col > ncols) ncols = col;
    };

    /* Now allocate the necessary memory for the rows */
    p->pat = (char **) G_malloc (nrows * sizeof(char *));

    /* and store the rows */
    for (row=0; row<nrows; row++)
      {
        p->pat[row] =  G_malloc(ncols);

        strncpy(p->pat[row], lines[row], ncols);

        for (i=0; i<ncols; i++)
          p->pat[row][i] -= '0'; 
     };

    p->nrows = nrows;
    p->ncols = ncols;
    add_pattern_to_list(name,p);
}


/* 
  Print pattern of category cat at current location
  Does not change graphics state
*/ 

print_pattern(CELL cat, double width, double height)
{
  int cells_wide = (int) width+0.5;
  int cells_high = (int) height+0.5;
  int doing_color;
  int r,g,b;
  int cc, row, col;

  doing_color=(PS.grey == 0 && PS.level == 2); 

  /* save graphics state */
  fprintf(PS.fp,"gsave\n");

  /* Tell postscript the width and height of our image */
  fprintf(PS.fp,"/cw %d def /ch %d def\n", cells_wide*2, cells_high*2); 

  /* Lower left corner of our image is : */
  fprintf(PS.fp, "currentpoint TR\n"); 

  /* Scale our image */
  fprintf(PS.fp, "%d %d scale\n", cells_wide, cells_high);

  /* make string which can hold one line of data */
  if (doing_color) fprintf(PS.fp, "/imgstrg cw 3 mul string def\n");
  else fprintf(PS.fp, "/imgstrg cw string def\n");

  /* Start the image operation */
    fprintf(PS.fp, "cw ch 8\n");
    fprintf(PS.fp, "[cw 0 0 ch neg 0 ch]\n");
    fprintf(PS.fp, "{currentfile imgstrg readhexstring pop}\n");
    if (doing_color) fprintf(PS.fp, "false 3 colorimage\n");
    else	     fprintf(PS.fp, "image\n");

   cc=0;
   for (row=0; row<cells_high*2; row++)
     for (col=0; col<cells_wide*2; col++)
       { 
       get_color(cat, &r, &g, &b, &PS.colors, row, col); 

       if (doing_color) 
          {
          fprintf(PS.fp,"%02x%02x%02x",r,g,b);
          cc+=6;
          }  
       else
          {
          fprintf(PS.fp, "%02x", (int) (0.3*r+0.59*g+0.11*b) );
	  cc+=2;
          };
       if (cc > 77) {cc=0; fprintf(PS.fp,"\n");};
       };
   
  
  /* restore graphics state */

  fprintf(PS.fp, "grestore\n");

}


#if 0

get_pattern (name, p)
    char *name;
    PATTERN *p;
{
    long offset;
    int nlines, nrows, ncols;
    int file;

    if (!find_pattern(name, &offset, &nlines, &nrows, &ncols, &file))
	return 0;

    p->nrows = nrows;
    p->ncols = ncols;

    open_pattern_file (file, 0);
    if (patternfd == NULL)
	return 0;

    fseek (patternfd, offset, 0);
    read_pattern (p, nlines);

    return 1;
}

begin_pattern (name)
    char *name;
{
    long ftell();


    open_pattern_file (TEMP_PATTERNS, 1);
    if (patternfd)
    {
	fflush (patternfd);
	fseek (patternfd, 0L, 2);
	cur_offset = ftell (patternfd);
	cur_name = G_store (name);
	nl = nr = nc = 0;
    }
    return 1;
}

store_pattern (buf)
    char *buf;
{
    int len;
    int n, color;

#ifdef DEBUG
printf ("store_pattern(%s)\n", buf);
#endif
    if (patternfd)
    {
#ifdef DEBUG
printf ("  checking color\n");
#endif
	switch (scan_patcolor (buf, &n, &color))
	{
	case -1: return -1;
	case  1: fprintf (patternfd, "%s\n", buf);
#ifdef DEBUG
printf ("  got color(%d) for cat(%d)\n", color, n);
#endif
		 nl++;
		 return 1;
	}
	len = strlen (buf);
	fprintf (patternfd, "%s\n", buf);
	nl++;
	nr++;
	if (len > nc)
	    nc = len;
    }
    return 1;
}

end_pattern ()
{
    if (patternfd)
    {
	fflush (patternfd);
	add_pattern_to_list ((char *)NULL, cur_offset, nl, nr, nc, cur_file);
    }
}

any_patterns()
{
    return plist != NULL;
}

/* this function treats the pattern list as a circular list, ie when
   the end of the list is reached it starts again at the beginning.
*/
next_pattern (pat,first)
    PATTERN *pat;
{
    static PLIST *p ;

    if (plist == NULL)	/* any patterns? */
	return 0;
    if (first)
	p = plist;
    pat->nrows = p->nrows;
    pat->ncols = p->ncols;

    if (!open_pattern_file (p->file, 0))
	return 0;

    fseek (patternfd, p->offset, 0);
    read_pattern (pat, p->nlines);

    p = p->next;
    if (p == NULL)
	p = plist;
    return 1;
}

open_pattern_file(file,mode)
{
    char buf[1024];
    char *name;
    if (patternfile == NULL)
    {
	patternfile = G_tempfile();
#ifdef DEBUG
printf ("pattternfile is %s\n", patternfile);
#endif
	patternfd = fopen(patternfile,"w");
	if (patternfd == NULL)
		G_fatal_error ("no temp files");
	fclose(patternfd);
	patternfd = NULL;
	cur_mode = -1;
    }
    if (cur_mode == mode && file == cur_file)
	return 1;
    close_pattern_file();
    cur_file = file;
    cur_mode = mode;
    switch (file)
    {
    case GLOBAL_PATTERNS:
	sprintf (name = buf, "%s/etc/paint/patterns", G_gisbase());
	break;
    case USER_PATTERNS:
	sprintf (name = buf, "%s/patterns", G_home());
	break;
    case TEMP_PATTERNS:
	name = patternfile;
	break;
    }

    if(patternfd = fopen(name,mode?"a":"r"))
    {
	cur_mode = mode;
	return 1;
    }
    cur_mode = cur_file = -1;
    return 0;
}

close_pattern_file()
{
    if (patternfd)
	close (patternfd);
    patternfd = NULL;
    cur_file = -1;
    cur_mode = -1;
}

unlink_pattern_file()
{
    close_pattern_file();
    unlink (patternfile);
    patternfile = NULL;
}


#endif 


