/* Functions: get_color_number, get_color_rgb, color_name_is_ok, get_color_name
**
** Author: Paul W. Carlson	March 1992
*/

#include <stdio.h>
#include "ps_info.h"

#define	NUM_COLORS	100

static struct {
    char *name;
    float r, g, b;
} colors[NUM_COLORS] =
{
    "white",	1.00, 1.00, 1.00,
    "black",	0.00, 0.00, 0.00,
    "red",	1.00, 0.00, 0.00,
    "green",	0.00, 1.00, 0.00,
    "blue",	0.00, 0.00, 1.00,
    "yellow",	1.00, 1.00, 0.00,
    "magenta",	1.00, 0.00, 1.00,
    "cyan",	0.00, 1.00, 1.00,
    "aqua",     0.00, 0.75, 0.75,
    "grey",	0.75, 0.75, 0.75,
    "gray",	0.75, 0.75, 0.75,
    "orange",	1.00, 0.50, 0.00,
    "brown",	0.75, 0.50, 0.25,
    "purple",	0.50, 0.00, 1.00,
    "violet",	0.50, 0.00, 1.00,
    "indigo",   0.00, 0.50, 1.00,
};

get_color_number(char *color_name)
{
    int i;

    G_strip(color_name);
    lowercase(color_name);
    for (i = 0; (i < NUM_COLORS) && (NULL!=colors[i].name); i++)
       if (strcmp(color_name, colors[i].name) == 0) return i;
    return -1;
}

get_color_rgb(int color_number, float *r, float *g, float *b)
{
    if (color_number < 0 || color_number >= NUM_COLORS)
    {
    	*r = *g = *b = 0.0;
	return -1;
    }
    *r = colors[color_number].r;
    *g = colors[color_number].g;
    *b = colors[color_number].b;
    return 1;
}


/* Retuns 1 iff given color name can be found */
color_name_is_ok(char *color_name)
{
    if (-1 == get_color_number(color_name)) 
       return 0;
    return 1;
}



set_color_name(char *name, float R, float G, float B)
{
  int i;

  for (i = 0; i < NUM_COLORS; i++)
    if (NULL==colors[i].name) break;
 
  if (NUM_COLORS == i) 
     error("set_color_name","Too many colors defined","");

  colors[i].name = G_store(name);
  colors[i].r = R;
  colors[i].g = G;
  colors[i].b = B;
};


/* Returns a pointer to the name of a given color number */
char *
get_color_name(int color_number)
{
    int i;

    if (color_number < 0 || color_number >= NUM_COLORS) return (char *)NULL;
    return colors[color_number].name;
}


/* Prints Postscript code to set a specified color */
set_rgb_color(int color_number)
{
    float r, g, b;

    if (get_color_rgb(color_number, &r, &g, &b) < 0)
    {
	r = g = b = 0.0;
    }
    fprintf(PS.fp, "%.2f %.2f %.2f C\n", r, g, b);
}

