/* Function: ps_sitetable
**
** Author: Horst Raap	July 2000
*/

#include "ps_info.h"
#include "sitetable.h"
#include "sites.h"

extern int verbose;

ps_sitetable()
{
    char buf[512];
    char *ms;
    int num_cats;
    int color ;
    int i, j, k;
    int R, G, B;
    int center_cols;
    double t, l, r;
    double x1, x2, y, y1, dy, fontsize, tl;
    double col_width;
    struct Range range;
    double xo[50], yo[50];
    double s, x_off, y_off;
    int points;
    int pidx;
    FILE *icon_fp, *sites_fp;

    /* let user know what's happenning */
    if (verbose > 1)
    {
        printf("PS-MAP: creating site table  ...");
        fflush(stdout);
    }

    if (verbose > 2)
      printf("st.x=%lf, st.y=%lf\n",st.x,st.y);

    /* set font */
    fontsize = (double)st.fontsize;
    fprintf(PS.fp, "(%s) FN %.1lf SF\n", st.font, fontsize);

    /* set sitetable location */
    dy = 1.5 * fontsize;

    if (st.y <= -1.5)
       t=PS.cur_y;
    else if (st.y <= 0.0) t = PS.min_y;
         else t = 72.0 * ( PS.page_height - st.y);

    if (st.x <= 0.0) st.x = PS.left_marg;
    l = 72.0 * st.x + 0.5;
    if (st.width <= 0.0 || st.width > PS.page_width  - PS.right_marg - st.x)
        st.width = PS.page_width  - PS.right_marg - st.x;
    r  = l + 72.0 * st.width;
    col_width = st.width / (double)st.cols;

    /* read descriptions into PostScript array "a" */
    fprintf(PS.fp, "/a [\n");
    for (i=0; i<site.count; i++)
      {
	/* open the sites file */
	sites_fp = G_fopen_sites_old(site.name[i], site.mapset[i]);
	if (sites_fp == NULL)
	{
	    sprintf(buf,"%s in %s", site.name[i], site.mapset[i]);
	    error("site list", buf, "could not open");
	    continue;
	}
        fgets(buf,100,sites_fp);
        /* Line 2 should hold the description */
        /* commenting lines are not taken care of at the moment */
        fgets(buf,100,sites_fp);
        /* Now description should start at buf[5] */
        fprintf(PS.fp, "(%s)\n", &(buf[5]));

        fclose(sites_fp);
       };
    fprintf(PS.fp, "] def\n");

    /* get width of widest string in PostScript variable "mw" */
    fprintf(PS.fp, "/mw 0 def 0 1 a length 1 sub { /i XD\n");
    fprintf(PS.fp, "a i get SW pop /t XD t mw gt {/mw t def} if } for\n");

    /* shrink font size to fit in width */
    if(st.cols == 1) tl = 72.0 * col_width - 2.0 * fontsize; 
    else tl = 72.0 * col_width - 4.0 * fontsize; 
    fprintf(PS.fp, "/s %1.lf def\n", fontsize);
    fprintf(PS.fp, "mw %.1lf gt {/s s %.1lf mul mw div def } if\n", tl, tl);
    fprintf(PS.fp, "(%s) FN s SF\n", st.font);

    /* make proc to center multiple columns */
    center_cols = (st.cols > 1);
    center_cols = 0; /* Just works o.k. this way */

    if (center_cols)
    {
    	fprintf(PS.fp, "/k %d def\n", st.cols - 1);
    	fprintf(PS.fp, "/mlw 0 def 0 k a length 1 sub { /i XD\n");
    	fprintf(PS.fp, "a i get SW pop /t XD t mlw gt {/mlw t def} if } for\n");
	fprintf(PS.fp, "/xo mw mlw sub D2 s mul %1.lf div %1.lf add def\n", 
		fontsize, fontsize);
	fprintf(PS.fp, "/mvx {xo add} BD\n");
    }

    y = t - fontsize;
    k = 0;
    for(i = 0; i < site.count; )
    {
	y -= dy;
	/* test for bottom of page */
	if (y < 72.0 * PS.bot_marg) 
	{
	    y = 72.0 * (PS.page_height - PS.top_marg) - 0.5 * fontsize;
	    fprintf(PS.fp, "showpage\n");
	};

	for (j = 0; (j<st.cols) && (i<site.count); j++, i++)
	{
 	  /* read icon file */
	  ms= G_find_file ("ps_icons", site.icon[i], "");
 
          if ((icon_fp = G_fopen_old("ps_icons", site.icon[i], ms)) 
		!= NULL) 
          {
	    /* read past icon title */
	    fgets(buf, 100, icon_fp);

	    /* read coordinates */
	    points = 0;
    	    while (fscanf(icon_fp, "%lf %lf", &x_off, &y_off) == 2)
	    {
		xo[points] = x_off;
		yo[points] = y_off;
	    	points++;

	    }
	    fclose(icon_fp);
          }
          else
	  /* use default icon (diamond) */
          {
	    points = 4;
	    xo[0] =  0.0;  yo[0] =  0.5;
	    xo[1] = -0.5;  yo[1] =  0.0;
	    xo[2] =  0.0;  yo[2] = -0.5;
	    xo[3] =  0.5;  yo[3] =  0.0;
	  };

	  /* draw icon i at column j*/
	  set_rgb_color(site.color[i]);
          s = fontsize;
	  x1 = l + (double)j * 72.0 * col_width + 0.5 * fontsize;
          y1= y + 0.5 * fontsize;
          if (verbose > 2)
            printf("st.x=%lf, st.y=%lf\n",x1,y1);
	  fprintf(PS.fp, "%.1lf ", x1 + s * xo[0]);
    	  if (center_cols) fprintf(PS.fp, "mvx ");
	  fprintf(PS.fp, "%.1lf NM\n", y1 + s * yo[0]);

	  for (pidx = 1; pidx < points; pidx++) 
            {
	    fprintf(PS.fp, "%.1lf ", x1 + s * xo[pidx]);
    	    if (center_cols) fprintf(PS.fp, "mvx ");
	    fprintf(PS.fp, "%.1lf LN\n", y1+ s * yo[pidx]);
            };
          fprintf(PS.fp, "CF\n");


	    /* do the text */
            set_rgb_color(st.color);
            fprintf(PS.fp, "a %d get %.1lf ", k++, x1 + 1.0 * fontsize);
    	    if (center_cols) fprintf(PS.fp, "mvx ");
            fprintf(PS.fp, "%.1lf MS\n", y);

       };


    }
    y -= dy;
    PS.cur_y=y;
    if (PS.min_y > y) PS.min_y = y;

    if (verbose > 1) printf("\n");
}

