/* Routine to scan "where" - commands 
   A number without specification is taken to be an absolute length (in inch) measured from top/left margin of printer
   A value with trailing "cm" is supposed to be absolute in centimeters as above
   A value with trailing "%" is interpreted as percentage of map coordinates

*/

#include "ps_info.h"

extern int scan_percent (char *buf, double *f, double min, double max);

/* 
  Input:
     data - string to scan
     x,y - locations to store the result

  Output:
    returns 0 if everything ok.
    returns -1 if failure
*/


int
scan_where( char *data, double *px, double *py)
  {
   char str1[100], str2[100];

   /* Check for illegal length */
   if (strlen(data) > 100)
     {
     error("where", data, "illegal where request");
     return -1;
     };

   sscanf(data, "%s %s", str1, str2);

   if (0 == getsize(str1, px, 0) &&  0 == getsize(str2, py, 1))
     return 0;

  *px = *py = -1.0;
  error("where", data, "illegal where request");
  return -1;
  };



/* scan a single numerical description
 * if "cm" is appended, convert from cm to inch
 * parameter fHeight is <> 0 iff height value is scanned

 * Returns 0 if successful
 * Changes parameter pnum
*/
static int
getsize (char * s, double *pnum, int fHeight)
  {
  double factor = 1.0;   
  double max;

  if (0==fHeight) max=PS.map_width;
  else max=PS.map_height;

  if (scan_percent(s, pnum, (double) 0.0, max))
    {   
    /* We have a valid percentage 
       Adjust with map offset
    */
    if (0==fHeight) *pnum += PS.map_x_orig;
    else *pnum += (PS.page_height-PS.map_y_orig);
    return 0;
    };


  if (0 == strcmp("cm", &(s[strlen(s)-2])) )
    {
    factor = 2.54;
    /* CHECK: Bad style, do not change s !!!*/
    s[strlen(s)-2] = '\0';
    };
  if (1 != sscanf(s, "%lf", pnum))
         return -1;
  
  if (*pnum <= 0.0) return 0;
  *pnum /= factor;
  if (0==fHeight) *pnum += PS.left_marg;
  else *pnum += PS.top_marg;

  return 0;
  };
      
