/* Function: vect_legend
**
** Author: Paul W. Carlson	April 1992
*/

#include "vector.h"
#include "ps_info.h"

vect_legend()
{
    int i, j, center_cols;
    char buf[100];
    double x, y, k, fontsize, dy, margin;
    double col_width, r, l, tl;
    double linelength; 
    struct Categories cats; 
    int cat; 

    /* set font */
    fontsize = (double)vector.fontsize;
    fprintf(PS.fp, "(%s) FN %.1lf SF\n", vector.font, fontsize);

    linelength = 5.0 * fontsize;

    /* get text location */
    dy = 1.5 * fontsize;
    if (vector.x > 0.0) x = 72.0 * vector.x;
    else x = PS.map_left;

    /* Set y-position */
    if (vector.y <= -1.5) y = PS.cur_y;
    else if (vector.y <= 0.0) y = PS.min_y;
         else y = 72.0 * ( PS.page_height - vector.y);

    margin = 0.4 * fontsize;
    if (x < PS.map_left + margin) x = PS.map_left + margin;

    /* set vectortable location */
    dy = 1.5 * fontsize;

    l = 72.0 * vector.x + 0.5;

    fprintf(PS.fp,"gsave\n");

    vector.tablewidth = PS.page_width  - PS.right_marg - vector.x;
    r  = l + 72.0 * vector.tablewidth;

    col_width = vector.tablewidth / (double)vector.cols;

    /* make PostScript array "a" of description strings */
    fprintf(PS.fp, "/a [\n");
    for (i = 0; i < vector.count; i++)
    {
      if (vector.line_cat[i] > 0)
        cat=vector.line_cat[i];
      else if (vector.area_cat[i] > 0)
        cat=vector.area_cat[i];
      else cat = 1;  /* We should probably get the title here !!! */ 
      G_read_vector_cats(vector.name[i], vector.mapset[i], &cats);
      fprintf(PS.fp, "( %s )\n",G_get_cat(cat, &cats));
    }
    fprintf(PS.fp, "] def\n");


    /* get width of widest string in PostScript variable "mw" */
    fprintf(PS.fp, "/mw 0 def 0 1 a length 1 sub { /i XD\n");
    fprintf(PS.fp, "a i get SW pop /t XD t mw gt {/mw t def} if } for\n");

    /* shrink font size to fit in width */
    if(vector.cols == 1) tl = 72.0 * col_width - 1.0 * fontsize; 
    else tl = 72.0 * col_width - 2.0 * fontsize; 
    fprintf(PS.fp, "/s %1.lf def\n", fontsize);
    fprintf(PS.fp, "mw %.1lf gt {/s s %.1lf mul mw div def } if\n", tl, tl);
    fprintf(PS.fp, "(%s) FN s SF\n", vector.font);

    /* make proc to center multiple columns */
    center_cols = (vector.cols > 1);
    center_cols = 0; /* Just works better this way */

    if (center_cols)
    {
    	fprintf(PS.fp, "/k %d def\n", vector.cols - 1);
    	fprintf(PS.fp, "/mlw 0 def 0 k a length 1 sub { /i XD\n");
    	fprintf(PS.fp, "a i get SW pop /t XD t mlw gt {/mlw t def} if } for\n");
	fprintf(PS.fp, "/xo mw mlw sub D2 s mul %1.lf div %1.lf add def\n", 
		fontsize, fontsize);
	fprintf(PS.fp, "/mvx {xo add} BD\n");
    }


    for (i = 0; i < vector.count;i++)
    {
	y -= dy;

	/* test for bottom of page */
	if (y < 72.0 * PS.bot_marg) 
	{
	    y = 72.0 * (PS.page_height - PS.top_marg) - 0.5 * fontsize;
	    fprintf(PS.fp, "showpage\n");
	};

	for (j = 0; (j<vector.cols) && (i<vector.count); j++, i++)
	{

          /* draw vector i at column j */
          /* starting x-coordinate */
          x = l + 72.0 * (double) j * col_width;
 
    	/* make white or grey background for lines and text */
        if ((vector.hwidth[i] > 0. && vector.hcolor[i] == WHITE) ||
	    (vector.hwidth[i] < 1. && vector.colors[i][0]  == WHITE))
           set_rgb_color(GREY);
        else 
           set_rgb_color(WHITE);

    	fprintf(PS.fp, "%.1lf ",x);
        if (center_cols) fprintf(PS.fp, "mvx ");
        fprintf(PS.fp, "%1lf ", y - 0.25 * dy);
        fprintf(PS.fp, "mw %.1lf add ", x + linelength + 1.0 * fontsize);
        if (center_cols) fprintf(PS.fp, "mvx ");
        fprintf(PS.fp, "%.1lf B fill \n", y+dy);


	/* do highlight, if any */
	if (vector.hwidth[i])
	{
	    set_rgb_color(vector.hcolor[i]);
	    fprintf(PS.fp, "%.8f W\n",  
		  vector.width[i] + 2 * vector.hwidth[i]);
	    fprintf(PS.fp, "[] 0 setdash\n");
            fprintf(PS.fp, "%.1lf ",x);
            if (center_cols) fprintf(PS.fp, "mvx ");
            fprintf(PS.fp, "%.1lf ",y + 0.5 * fontsize);
            fprintf(PS.fp, "%.1lf ",x + linelength);
            if (center_cols) fprintf(PS.fp, "mvx ");
            fprintf(PS.fp, "%.1lf L\n",y + 0.5 * fontsize);
	}

	/* plot the primary color line */
	set_rgb_color(vector.colors[i][0]);
	fprintf(PS.fp, "%.8f W\n", vector.width[i] );
	fprintf(PS.fp, "%s setdash\n", vector.setdash[i]);

        fprintf(PS.fp, "%.1lf ",x);
        if (center_cols) fprintf(PS.fp, "mvx ");
        fprintf(PS.fp, "%.1lf ",y + 0.5 * fontsize);
        fprintf(PS.fp, "%.1lf ",x + linelength);
        if (center_cols) fprintf(PS.fp, "mvx ");
        fprintf(PS.fp, "%.1lf L\n",y + 0.5 * fontsize);


	/* plot the text */
	set_rgb_color(BLACK);
        fprintf(PS.fp, "a %d get %.1lf ", i, x + linelength + 1.0 * fontsize);
        if (center_cols) fprintf(PS.fp, "mvx ");
        fprintf(PS.fp, "%.1lf MS\n", y);
       }; 
    }; 

    PS.cur_y=y; 
    if (PS.min_y > y) PS.min_y = y;
    fprintf(PS.fp,"grestore\n");
}

